"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _invariant = require("invariant");

var _invariant2 = _interopRequireDefault(_invariant);

var _utils = require("../../utils");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.default = function (_ref) {
  var client = _ref.client;
  return function (urls) {
    var folderid = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
    var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

    (0, _invariant2.default)(urls, "`file` is required.");
    (0, _invariant2.default)(typeof urls === "string", "`file` must be supplied");

    var _options$onBegin = options.onBegin,
        onBegin = _options$onBegin === undefined ? function () {} : _options$onBegin,
        _options$onProgress = options.onProgress,
        onProgress = _options$onProgress === undefined ? function () {} : _options$onProgress,
        _options$onFinish = options.onFinish,
        onFinish = _options$onFinish === undefined ? function () {} : _options$onFinish;

    var progressId = "pcloud-sdk-remote-" + (0, _utils.uniqueNumber)() + "-" + (0, _utils.randomNumber)();
    var progressTimeout = void 0;

    var progress = function progress(apiServer) {
      pollProgress(apiServer);
      progressTimeout = setTimeout(function () {
        return progress(apiServer);
      }, 200);
    };

    var stopProgress = function stopProgress() {
      if (progressTimeout) {
        clearTimeout(progressTimeout);
      }
    };

    var pollProgress = function pollProgress(apiServer) {
      client.api("uploadprogress", { params: { progresshash: progressId, apiServer: apiServer } }).then(function (_ref2) {
        var files = _ref2.files;
        return onProgress(calculateProgress(files));
      }).catch(function (_ref3) {
        var result = _ref3.result,
            error = _ref3.error;

        if (result === 1900) {
          onProgress(calculateProgress());
        }
        console.log(error);
      });
    };

    onBegin();

    return client.api("currentserver").then(function (_ref4) {
      var hostname = _ref4.hostname;


      var promise = client.api("downloadfile", {
        method: "post",
        params: {
          folderid: folderid,
          progresshash: progressId,
          nopartial: 1,
          url: urls,
          apiServer: hostname
        }
      });

      progress(hostname);
      return promise;
    }).then(function (_ref5) {
      var metadata = _ref5.metadata;

      stopProgress();

      onFinish({ metadata: metadata[0] });
      return { metadata: metadata[0] };
    }).catch(function (err) {
      return console.log("err", err);
    });
  };
};

var calculateProgress = function calculateProgress() {
  var files = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];

  return {
    all: {
      downloaded: files.reduce(function (n, _ref6) {
        var _ref6$downloaded = _ref6.downloaded,
            downloaded = _ref6$downloaded === undefined ? 0 : _ref6$downloaded;
        return n + downloaded;
      }, 0),
      size: files.reduce(function (n, _ref7) {
        var _ref7$size = _ref7.size,
            size = _ref7$size === undefined ? 0 : _ref7$size;
        return n + size;
      }, 0)
    },
    files: files
  };
};